from collections import defaultdict

from charms.reactive import RelationBase
from charms.reactive import hook
from charms.reactive import scopes

from charmhelpers.contrib.templating.contexts import dict_keys_without_hyphens


class SlurmProvides(RelationBase):
    scope = scopes.UNIT

    @hook('{provides:slurm-cluster}-relation-{joined,changed}')
    def joined_or_changed(self):
        self.set_state('{relation_name}.connected')
        if self.nodes_ready() and self.get_partitions():
            self.set_state('{relation_name}.available')
        else:
            self.remove_state('{relation_name}.available')

    @hook('{provides:slurm-cluster}-relation-{departed,broken}')
    def departed(self):
        self.remove_state('{relation_name}.connected')
        self.remove_state('{relation_name}.available')

    def nodes_ready(self):
        if self.get_nodes():
            for node in self.get_nodes():
                if not all(node.values()):
                    return False
            return True
        return False

    def set_config(self, config):
        data = {'config': config}
        for conv in self.conversations():
            conv.set_remote(**data)

    def get_nodes(self):
        return list(self._generate_nodes())

    def get_partitions(self):
        partitions_dict = defaultdict(list)
        for node in self.get_nodes():
            partitions_dict[node['partition']].append(node['hostname'])
        return dict(partitions_dict)

    def _generate_nodes(self):
        for conv in self.conversations():
            yield dict_keys_without_hyphens({
                'hostname': conv.get_remote('hostname'),
                'private-address': conv.get_remote('private-address'),
                'partition': conv.get_remote('partition')
            })
