from charms.reactive import when
from charms.reactive import when_not
from charms.reactive import set_state
from charms.reactive import only_once
from charms.reactive import remove_state
from charms.reactive.helpers import data_changed

from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core.templating import render

SLURM_CONFIG_TEMPLATE = 'slurm.conf'
SLURM_CONFIG_PATH = '/etc/slurm-llnl/slurm.conf'
MUNGE_KEY_TEMPLATE = 'munge.key'
MUNGE_KEY_PATH = '/etc/munge/munge.key'


@only_once()
@when('slurm.installed')
def initial_setup():
    hookenv.status_set('maintenance', 'Initial setup of slurm-node')
    # Make sure slurmctld is disabled on node
    set_state('slurmctld.disable')


@when_not('slurm-cluster.connected', 'slurm-cluster.available')
def waiting_on_controllers():
    set_state('slurmd.stop')
    hookenv.status_set('blocked', 'Missing relation to slurm-controller')


@when('slurm-cluster.connected')
def update_node_info(cluster):
    cluster.set_node_info()


@when('slurm-cluster.available')
def configure_node(cluster):
    controller_config = cluster.get_config()
    if data_changed('config', controller_config):
        remove_state('slurm-node.configured')
        hookenv.status_set('maintenance', 'Configuring slurm-node')

        render_munge_key(controller_config)
        set_state('munge.restart')

        render_slurm_config(controller_config)
        create_spool_dir(controller_config)
        set_state('slurmd.restart')
        set_state('slurm-node.configured')


@when('slurm-cluster.available', 'slurm-node.configured')
def node_ready(cluster):
    hookenv.status_set('active', 'Ready')


def render_slurm_config(config):
    render(source=SLURM_CONFIG_TEMPLATE,
           target=SLURM_CONFIG_PATH,
           context=config,
           owner=config.get('slurm_user'),
           group=config.get('slurm_user'),
           perms=0o644)


def render_munge_key(config):
    render(source=MUNGE_KEY_TEMPLATE,
           target=MUNGE_KEY_PATH,
           context=config,
           owner='munge',
           group='munge',
           perms=0o400)


def create_spool_dir(config):
    host.mkdir(path=config.get('slurmd_spool_dir'),
               owner=config.get('slurm_user'),
               group=config.get('slurm_user'),
               perms=0o750)
